/* SigmaStar trade secret */
/*
* cmd_sigauth.c - Sigmastar
*
* Copyright (C) 2018 Sigmastar Technology Corp.
*
* Author: raul.wang <raul.wang@sigmastar.com.tw>
*
* This software is licensed under the terms of the GNU General Public
* License version 2, as published by the Free Software Foundation, and
* may be copied, distributed, and modified under those terms.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
*/

#include <common.h>
#include <command.h>
#include <malloc.h>
#include "asm/arch/mach/ms_types.h"
#include "asm/arch/mach/platform.h"
#include "asm/arch/mach/io.h"

#include <../drivers/mstar/aesdma/drvAESDMA.h>
#include <rtk.h>

extern void halt(void);

#define IPL_HEADER_OFFSET               0x04
#define IPL_SIZE_OFFSET                 0x08
#define IPL_CUST_LENGTH_N               0x10
#define IPL_CUST_LENGTH_AES             0x12
#define IPL_CUST_MAKER                  0x434c5049

int do_sig_auth(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    u32 image_start = 0;
    u32 image_size = 0;
    u32 key_start = 0;
    u32 KeyN = 0;

    if (argc < 3)
	{
		printf("missing parameters\n");
		return CMD_RET_USAGE;
	}

    image_start = (u32)simple_strtoul(argv[1], NULL, 16);
    key_start = (u32)simple_strtoul(argv[2], NULL, 16);

    if (0 == key_start)
    {
        KeyN = 0;
    }
    else if (IPL_CUST_MAKER == *(U32*)(key_start + IPL_HEADER_OFFSET))
    {
        u16 u16LenN = 0;
        u16 u16LenAES = 0;
        u16 u16LenIPLCust = 0;

        printf("[U-Boot] Key in IPL_CUST\n\r");
        u16LenN = *(volatile U16*)(key_start + IPL_CUST_LENGTH_N);
        u16LenAES = *(volatile U16*)(key_start + IPL_CUST_LENGTH_AES);
        u16LenIPLCust = *(volatile U16*)(key_start + IPL_SIZE_OFFSET);
        KeyN = (u32) (key_start + (u16LenIPLCust - (u16LenN + u16LenAES)));
    }
    else
    {
        printf("OTP\n\r");
        KeyN = key_start;
    }

    if (RTK_MAKER == image_get_rtk_maker(image_start))
    {
        image_size = image_get_rtk_size(image_start);
    }
    else if (image_check_magic((const image_header_t*) image_start))
    {
        image_size = image_get_data_size((const image_header_t*) image_start);
        image_start += image_get_header_size();
    }

    if (runAuthenticate(image_start, image_size, (U32*)KeyN))
    {
        printf("[U-Boot] Authenticate pass!\n\r");
    }
    else
    {
        printf("[U-Boot] Authenticate failed!\n\r");
        halt();
    }

    return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	sigauth, CONFIG_SYS_MAXARGS, 1,	do_sig_auth,
	"Only verify digital signature without aes",
	"\n"
);

