/*
 * Copyright 2008 - 2009 Windriver, <www.windriver.com>
 * Author: Tom Rix <Tom.Rix@windriver.com>
 *
 * (C) Copyright 2014 Linaro, Ltd.
 * Rob Herring <robh@kernel.org>
 *
 * Portions Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */
#include <common.h>
#include <command.h>

#ifdef CONFIG_CMD_FASTBOOT
#include <g_dnl.h>
#include <fastboot.h>
#include <watchdog.h>

int fastboot_try(unsigned long timeout_ms)
{
	int ret;
	int auto_detach = 0;
	unsigned long start_ms = 0;

	g_dnl_clear_detach();
	ret = g_dnl_register("usb_dnl_fastboot");
	if (ret)
		return ret;

	fastboot_init((void *)0, 0);	/* Ignore any arguments, select defaults */

	if (timeout_ms) {
		auto_detach = 1;
		start_ms = get_timer(0);
	}

	while (1) {
		if (g_dnl_detach())
			break;
		if (ctrlc())
			break;
		if (auto_detach) {
			g_dnl_trigger_detach();
		}
		usb_gadget_handle_interrupts();
		if (auto_detach) {
			if (g_dnl_detach()) {
				if (get_timer(start_ms) < timeout_ms)
					g_dnl_clear_detach();
			} else
				auto_detach = 0;
		}
		WATCHDOG_RESET();
	}

	g_dnl_unregister();
	g_dnl_clear_detach();
	return CMD_RET_SUCCESS;
}

static int do_fastboot(cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
	unsigned long timeout_ms = 0;
	if (argc == 3 && (strcmp(argv[1], "try") == 0)) {
		/* timeout in milliseconds should be passed as an argument */
		timeout_ms = simple_strtoul(argv[2], NULL, 10);
	}
	return fastboot_try(timeout_ms);
}


U_BOOT_CMD(
	fastboot,	3,	0,	do_fastboot,
	"use USB Fastboot protocol",
	"\n"
	"    - run as a fastboot usb device\n"
 	"Usage:\n"
 	"fastboot [try TIMEOUT_MS]\n"
);

#endif

#ifdef CONFIG_CMD_USBSTART
#include <net.h>

static int do_usbstart(cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
    usb_eth_initialize(NULL);
    return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	usbstart,	1,	0,	do_usbstart,
	"Start USB Device",
	"reset  - reset USBD controller\n"
	"start  - start USBD controller\n"
);

#endif


