/*
* drvAESDMA.c- Sigmastar
*
* Copyright (C) 2018 Sigmastar Technology Corp.
*
* Author: nick.lin <nick.lin@sigmastar.com.tw>
*
* This software is licensed under the terms of the GNU General Public
* License version 2, as published by the Free Software Foundation, and
* may be copied, distributed, and modified under those terms.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
*/


#ifndef _DRV_AESDMA_H_
#include "drvAESDMA.h"
#endif
#include <common.h>
#include "asm/arch/mach/io.h"
#include "asm/arch/mach/platform.h"


void MDrv_AESDMA_Run(aesdmaConfig *pConfig)
{
    HAL_AESDMA_Reset();

    HAL_AESDMA_SetFileinAddr(pConfig->u32SrcAddr);
    HAL_AESDMA_SetXIULength(pConfig->u32Size);
    HAL_AESDMA_SetFileoutAddr(pConfig->u32DstAddr, pConfig->u32Size);

    switch(pConfig->eKeyType)
    {
    case E_AESDMA_KEY_CIPHER:
        HAL_AESDMA_UseCipherKey();
        HAL_AESDMA_SetCipherKey(pConfig->pu16Key);
        break;
    case E_AESDMA_KEY_EFUSE:
        HAL_AESDMA_UseEfuseKey();
        break;
    case E_AESDMA_KEY_HW:
        HAL_AESDMA_UseHwKey();
        break;
    default:
        return;
    }

    if(pConfig->bDecrypt)
    {
        HAL_AESDMA_CipherDecrypt();
    }
    else
    {
        HAL_AESDMA_CipherEncrypt();
    }

    if(pConfig->bSetIV)
    {
        HAL_AESDMA_SetIV(pConfig->pu16IV);
    }

    HAL_AESDMA_Enable();

    switch(pConfig->eChainMode)
    {
    case E_AESDMA_CHAINMODE_ECB:
        HAL_AESDMA_SetChainModeECB();
        HAL_AESDMA_SetXIULength(((pConfig->u32Size+15)/16)*16); // ECB mode size should align 16byte
        break;
    case E_AESDMA_CHAINMODE_CTR:
        HAL_AESDMA_SetChainModeCTR();
        HAL_AESDMA_CipherEncrypt();  // CTR mode can't set cipher_decrypt bit
        break;
    case E_AESDMA_CHAINMODE_CBC:
        HAL_AESDMA_SetChainModeCBC();
        break;
    default:
        return;
    }

    HAL_AESDMA_FileOutEnable(1);

    HAL_AESDMA_Start(1);

    // Wait for ready.
    while((HAL_AESDMA_GetStatus() & AESDMA_CTRL_DMA_DONE) != AESDMA_CTRL_DMA_DONE);
    HAL_AESDMA_Reset();

}

void MDrv_SHA_Run(U32 u32SrcAddr, U32 u32Size, enumShaMode eMode, U16 *pu16Output)
{
    HAL_SHA_Reset();

    HAL_SHA_SetAddress(u32SrcAddr);
    HAL_SHA_SetLength(u32Size);

    switch(eMode)
    {
    case E_SHA_MODE_1:
        HAL_SHA_SelMode(0);
        break;
    case E_SHA_MODE_256:
        HAL_SHA_SelMode(1);
        break;
    default:
        return;
    }

    HAL_SHA_Start();

    // Wait for the SHA done.
    while((HAL_SHA_GetStatus() & SHARNG_CTRL_SHA_READY) != SHARNG_CTRL_SHA_READY);

    HAL_SHA_Out((U32)pu16Output);

    HAL_SHA_Clear();
    HAL_SHA_Reset();
}

void MDrv_RSA_Run(rsaConfig *pConfig)
{
    int nOutSize;
    int i;

    HAL_RSA_Reset();

    HAL_RSA_SetKeyLength((pConfig->u32KeyLen-1) & 0x3F);
    HAL_RSA_SetKeyType(pConfig->bHwKey, pConfig->bPublicKey);

    HAL_RSA_Ind32Ctrl(0);
    HAL_RSA_Ind32Ctrl(1);
    HAL_RSA_LoadSignInverse_2byte((U16*)pConfig->pu32Sig);
    if (!pConfig->bHwKey)
    {
        if (pConfig->pu32KeyN)
        {
            HAL_RSA_LoadKeyNInverse(pConfig->pu32KeyN);
        }
        HAL_RSA_LoadKeyE(pConfig->pu32KeyE);
    }

    HAL_RSA_ExponetialStart();

    while((HAL_RSA_GetStatus() & RSA_STATUS_RSA_DONE) != RSA_STATUS_RSA_DONE);

    if((pConfig->bHwKey) || (pConfig->u32KeyLen == 2048))
    {
        nOutSize = (2048/8)/4;
    }
    else
    {
        nOutSize = (1024/8)/4;
    }

    HAL_RSA_Ind32Ctrl(0);
    for( i = 0; i<nOutSize; i++)
    {
        HAL_RSA_SetFileOutAddr(i);
        HAL_RSA_FileOutStart();
        *(pConfig->pu32Output+i) = HAL_RSA_FileOut();
    }
    HAL_RSA_FileOutEnd();
    HAL_RSA_Reset();
}


void runDecrypt(U32 u32ImageAddr, U32 u32ImageSize)
{
    aesdmaConfig config={0};

    config.u32SrcAddr=u32ImageAddr;
    config.u32DstAddr=u32ImageAddr;
    config.u32Size=u32ImageSize;
    //config.eKeyType=E_AESDMA_KEY_EFUSE;  //default use EFUSE key
    config.eKeyType=E_AESDMA_KEY_HW;  //use HW key
    config.bDecrypt=1;
    config.eChainMode=E_AESDMA_CHAINMODE_ECB;  //default use CBC mode
    MDrv_AESDMA_Run(&config);
   // printf("[U-Boot] Decrypt KEY_CUST done!\n\r");
}

BOOL runSHA256(U32 u32SrcAddr, U32 len, U32 *sha_out)
{
    OUTREG8(GET_REG_ADDR(REG_ADDR_BASE_CLKGEN, CKG_AESDMA), CKG_AESDMA_ENABLE);
    MDrv_SHA_Run(u32SrcAddr, len, E_SHA_MODE_256, (U16*)sha_out);
    return TRUE;
}

BOOL runRSA(U32 u32SigAddr, U32 *sha_sum, U32 *pu32Key)
{
    U32 rsa_out[64];
    rsaConfig config={0};
    U32 eValue = DEFAULT_EXP_VALUE;
    int compareOKcount = 0;
    static const int KNumOutWords=8;
    U8 i;

    OUTREG8(GET_REG_ADDR(REG_ADDR_BASE_CLKGEN, CKG_AESDMA), CKG_AESDMA_ENABLE);

    if(pu32Key == 0 || *pu32Key == 0)
    {
        config.bHwKey=1;
    }
    else
    {
        config.pu32KeyN = pu32Key;
        config.pu32KeyE = eValue;
        config.u32KeyLen = 2048;
    }
    config.bPublicKey=1;
    config.pu32Sig=(U32*)(u32SigAddr);
    config.pu32Output=rsa_out;

    MDrv_RSA_Run(&config);

    for (i=0; i<KNumOutWords; i++)
    {
        // add random delays to upset glitching attacks. ~1ns per instruction @ 1GHz so even 1us is massive
        srand(get_ticks() ^ rand());
        int del = rand() & 0xF;
        udelay(del);
        if (sha_sum[i] == rsa_out[i])
        {
            compareOKcount++;
        }
        else
        {
            goto failexit;
        }
    }
    if (compareOKcount == KNumOutWords) // make sure we didn't skip any comparisons
    {
        printf("%s: compare RSA address --> MDrv_RSA_Run done\n", __func__);
        return TRUE;
    }
failexit:
    printf("[U-Boot] RSA check failed.\n");
    return FALSE;
}

BOOL runAuthenticate(U32 u32ImageAddr, U32 u32ImageSize, U32* pu32Key)
{
    U32 sha_out[8];

    OUTREG8(GET_REG_ADDR(REG_ADDR_BASE_CLKGEN, CKG_AESDMA), CKG_AESDMA_ENABLE);

    MDrv_SHA_Run(u32ImageAddr, u32ImageSize, E_SHA_MODE_256, (U16*)sha_out); //image + SHA-256
    return runRSA(u32ImageAddr + u32ImageSize, sha_out, pu32Key);
}

