/* Copyright 2019 Amazon.com, Inc. or its affiliates.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <stdio.h>
#include <string.h>
#include <gst/gst.h>

GType gst_audio_sink_get_type ();
GType gst_video_meta_api_get_type ();

GType gst_queue_get_type ();
GType gst_capsfilter_get_type ();
GType gst_file_src_get_type ();
GType gst_fake_sink_get_type ();
GType gst_file_sink_get_type ();
GType gst_funnel_get_type ();
GType gst_identity_get_type ();
GType gst_tee_get_type ();

GType gst_opus_enc_get_type ();
GType gst_opus_dec_get_type ();
GType gst_rtp_opus_pay_get_type ();
GType gst_rtp_opus_depay_get_type ();
GType gst_rtp_h264_pay_get_type ();
GType gst_rtp_h264_depay_get_type ();
GType gst_mulawenc_get_type ();
GType gst_mulawdec_get_type ();
GType gst_rtp_pcmu_pay_get_type ();
GType gst_rtp_pcmu_depay_get_type ();
GType gst_audio_resample_get_type ();

GType gst_rtp_jitter_buffer_get_type ();
GType gst_rtp_pt_demux_get_type ();
GType gst_rtp_session_get_type ();
GType gst_rtp_funnel_get_type ();
GType gst_rtp_rtx_receive_get_type ();
GType gst_rtp_rtx_send_get_type ();
GType gst_rtp_rtx_queue_get_type ();

GType gst_srtp_enc_get_type ();
GType gst_srtp_dec_get_type ();

GType gst_input_selector_get_type ();

GType gst_push_src_get_type();
GType gst_base_sink_get_type();

/* The GStreamer SRTP plugin does the srtp_init in it's plugin
 * init, which we are not calling. So do the srtp_init here. */
void srtp_init();
__attribute__((constructor)) static void init() {
  srtp_init();
}

/* This struct is shared with other projects. If you change this,
 * be sure to update them as well */
struct element_t {
  const char *name;
  const char *plugin;
  GType (*get_type) (void);
  const char *dependencies[16];
  int registered;
};

/* This is the list of elements we use, as well as their
 * init functions. */
static struct element_t elements_list[] = {
  {"capsfilter", 0, gst_capsfilter_get_type},
  {"funnel", 0, gst_funnel_get_type},
  {"identity", 0, gst_identity_get_type},
  {"queue", 0, gst_queue_get_type},
  {"tee", 0, gst_tee_get_type},
  {"opusenc", 0, gst_opus_enc_get_type},
  {"opusdec", 0, gst_opus_dec_get_type},
  {"rtpopuspay", 0, gst_rtp_opus_pay_get_type},
  {"rtpopusdepay", 0, gst_rtp_opus_depay_get_type},
  {"rtph264pay", 0, gst_rtp_h264_pay_get_type},
  {"rtph264depay", 0, gst_rtp_h264_depay_get_type},
  {"mulawenc", 0, gst_mulawenc_get_type},
  {"mulawdec", 0, gst_mulawdec_get_type},
  {"rtppcmupay", 0, gst_rtp_pcmu_pay_get_type},
  {"rtppcmudepay", 0, gst_rtp_pcmu_depay_get_type},
  {"audioresample", 0, gst_audio_resample_get_type},

  {"rtpjitterbuffer", 0, gst_rtp_jitter_buffer_get_type},
  {"rtpptdemux", 0, gst_rtp_pt_demux_get_type},
  {"rtpsession", 0, gst_rtp_session_get_type},
  {"rtpfunnel", 0, gst_rtp_funnel_get_type},
  {"rtprtxreceive", 0, gst_rtp_rtx_receive_get_type},
  {"rtprtxsend", 0, gst_rtp_rtx_send_get_type},
  {"rtprtxqueue", 0, gst_rtp_rtx_queue_get_type},
  {"srtpenc", 0, gst_srtp_enc_get_type},
  {"srtpdec", 0, gst_srtp_dec_get_type},
  {"input-selector", 0, gst_input_selector_get_type},
  {"filesrc", 0, gst_file_src_get_type},
  {"fakesink", 0, gst_fake_sink_get_type},
  {"filesink", 0, gst_file_sink_get_type},
  {0, 0}
};
struct element_t *g_external_elements_list() {return elements_list;}

/* Defined to avoid dependency on libgsttag */
GstBuffer *
gst_tag_list_to_vorbiscomment_buffer (const GstTagList * list,
    const guint8 * id_data, const guint id_data_length,
    const gchar * vendor_string)
{
  GstBuffer *buffer;
  GstMapInfo info;
  guint8 *data;

  buffer = gst_buffer_new_and_alloc (id_data_length + 4 + 1 + 4 + 1);
  gst_buffer_map (buffer, &info, GST_MAP_WRITE);
  data = info.data;
  if (id_data_length > 0) {
    memcpy (data, id_data, id_data_length);
    data += id_data_length;
  }
  GST_WRITE_UINT32_LE (data, 1 /* vendor_len */);
  data += 4;
  *data++ = 0; /* vendor */
  GST_WRITE_UINT32_LE (data, 0);
  data += 4;
  *data = 1;
  gst_buffer_unmap (buffer, &info);

  return buffer;
}

gchar *
gst_tag_freeform_string_to_utf8 (const gchar * data, gint size,
    const gchar ** env_vars) {
  return g_strdup("");
}

/* Defined to avoid dependency on libgio / libgstnet */
void *gst_buffer_get_net_address_meta() {return NULL;}
void *gst_ntp_clock_new() {g_assert(FALSE); return NULL;}
void *gst_ptp_clock_new() {g_assert(FALSE); return NULL;}
int g_inet_address_equal() {g_assert(FALSE); return TRUE;}
void *g_inet_socket_address_get_address() {g_assert(FALSE); return NULL;}
const char *g_inet_address_to_string() {g_assert(FALSE); return g_strdup("0.0.0.0");}
int g_inet_socket_address_get_port() {g_assert(FALSE); return 0;}
void *g_socket_address_get_type() {g_assert(FALSE); return 0;}
void *g_inet_socket_address_get_type() {g_assert(FALSE); return 0;}

/* Defined to avoid dependency on libgmodule */
gboolean g_module_supported() {return FALSE;}
void g_module_error() {g_assert(FALSE);}
void g_module_close() {g_assert(FALSE);}
void g_module_open() {g_assert(FALSE);}
void g_module_make_resident() {g_assert(FALSE);}
void g_module_symbol() {g_assert(FALSE);}
int dladdr() {return 0;}

/* Avoid linking in the Glib regex support */
gboolean g_regex_match_simple (const gchar *pattern, const gchar *string,
    GRegexCompileFlags compile_options, GRegexMatchFlags match_options) {return TRUE;}
