/*
 * Copyright (c) 2019-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * This file defines the interface between Red and Blue. Included in multiple builds
 */

#ifndef __COCOA_BLUE_UART_BOOT_H__
#define __COCOA_BLUE_UART_BOOT_H__

#define COCOA_BLUE_UART_VERSION     6

typedef enum _UART_Boot_IR_Leds {
    UB_IR_LED_0 = 0,
    UB_IR_LED_10,
    UB_IR_LED_20,
    UB_IR_LED_50
} UART_Boot_IR_Leds;

typedef enum _UART_Boot_Night_Mode {
    UB_Day_Mode = 0,
    UB_Night_Mode,
    UB_Night_Colour_Mode
} UART_Boot_Night_Mode;

typedef enum _UART_Boot_Fps {
    UB_FPS30 = 0,
    UB_FPS24,
    UB_FPS15,
    UB_FPS8,
} UART_Boot_Fps;

typedef enum _UART_Boot_BitRate {
    UB_BitRate_2m = 0,
    UB_BitRate_1500k,
    UB_BitRate_1000k,
    UB_BitRate_750k,
    UB_BitRate_500k,
    UB_BitRate_400k,
    UB_BitRate_350k,
    UB_BitRate_300k,
} UART_Boot_BitRate;

typedef enum _UART_Boot_Resolution {
    UB_Res_1080P = 0,
    UB_Res_720P,
    UB_Res_360P
} UART_Boot_Resolution;

typedef enum _UART_Boot_Flicker {
    UB_FLICKER_DISABLE = 0,
    UB_FLICKER_60HZ = 1,
    UB_FLICKER_50HZ = 2,
    UB_FLICKER_AUTO = 3
} UART_Boot_Flicker;

typedef enum _UART_Boot_Reason {
    UB_Reason_Streaming = 0,
    UB_Reason_Motion = 1,
    UB_Reason_Earcon = 2,
    UB_Reason_Snapshot = 3
} UART_Boot_Reason;

#define UART_BOOT_CHECKSUM_SIZE     (8)
#define UART_BOOT_CHECKSUM_MASK     ( (1 << UART_BOOT_CHECKSUM_SIZE) - 1)

typedef enum _UART_Boot_SectionNum {
    UB_Section1 = 1,
    UB_Section2,
    UB_SectionMax
} UART_Boot_SectionNum;

typedef struct _uart_boot_section1 {
    unsigned int            ProtocolVersion : 8;
    unsigned int            SensorGain : 16;    // [1024, 32*1024] equivalent to 1x..32x gain
    unsigned int            IspGain : 18;       // [1024, (256*1024-1)] equivalent to 1x..255.9999x gain
    unsigned int            ExposureTime : 18;  // [1, 200000] us
    unsigned int            RedGain : 13;       // [1024, (8*1024)-1]
    unsigned int            BlueGain : 13;      // [1024, (8*1024)-1]
    unsigned int            GreenGain : 13;     // [1024, (8*1024)-1]
    UART_Boot_IR_Leds       IRLeds : 2;
    UART_Boot_Night_Mode    DayNightMode : 2;
    UART_Boot_Fps           SensorFrameRate : 2;
    UART_Boot_Flicker       AntiFlicker : 2;
    unsigned int            BitSum : UART_BOOT_CHECKSUM_SIZE;
} __attribute__((packed)) uart_boot_section1;

typedef struct _uart_boot_section2 {
    unsigned int            TimeSeconds : 32;
    unsigned int            TimeMilliseconds : 16;
    unsigned int            BitSum : UART_BOOT_CHECKSUM_SIZE;
    UART_Boot_Reason        BootReason:2;
    UART_Boot_Resolution    Resolution : 2;
    UART_Boot_Fps           EncFrameRate : 2;
    unsigned int            RegionsNum : 2;
    UART_Boot_BitRate       TargetBitRate : 3;

    // Regions given as ratio of total image size, with range 0-1023
    // 1023 represents 1.0 * image size on each axis
    unsigned int            region0Xratio : 10;
    unsigned int            region0Yratio : 10;
    unsigned int            region0Wratio : 10;
    unsigned int            region0Hratio : 10;

    unsigned int            region1Xratio : 10;
    unsigned int            region1Yratio : 10;
    unsigned int            region1Wratio : 10;
    unsigned int            region1Hratio : 10;

    unsigned int            RMSEnabled : 1;
    unsigned int            HighResYuv : 1;
} __attribute__((packed)) uart_boot_section2;

// TODO COCOA-2224 - Add static asserts for expected struct sizes

#endif //__COCOA_BLUE_UART_BOOT_H__
